@@ -31,7 +31,7 @@ extern struct timeval starttime;
 
 /* Global variable because no way to pass it as argument. Should not be used
  * elsewhere. */
-struct lldpd *scfg;
+static struct lldpd *scfg;
 
 static inline uint8_t
 swap_bits(uint8_t n)
@@ -43,7 +43,7 @@ swap_bits(uint8_t n)
   return  n;
 };
 
-struct lldpd_hardware*
+static struct lldpd_hardware*
 header_portindexed_table(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
@@ -85,7 +85,7 @@ header_portindexed_table(struct variable *vp, oid *name, size_t *length,
 #define TPR_VARIANT_IP   1
 #define TPR_VARIANT_MED_POLICY 2
 #define TPR_VARIANT_MED_LOCATION 3
-struct lldpd_hardware*
+static struct lldpd_hardware*
 header_tprindexed_table(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method, int variant)
 {
@@ -179,7 +179,7 @@ header_tprindexed_table(struct variable *vp, oid *name, size_t *length,
 }
 
 #ifdef ENABLE_DOT1
-struct lldpd_vlan*
+static struct lldpd_vlan*
 header_pvindexed_table(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
@@ -231,7 +231,7 @@ header_pvindexed_table(struct variable *vp, oid *name, size_t *length,
 	return pvlan;
 }
 
-struct lldpd_vlan*
+static struct lldpd_vlan*
 header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
     int exact, size_t *var_len, WriteMethod **write_method)
 {
@@ -325,6 +325,8 @@ header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
 #define LLDP_SNMP_LOCAL_DOT3_AUTONEG_MAU 8
 #define LLDP_SNMP_LOCAL_DOT3_AGG_STATUS 9
 #define LLDP_SNMP_LOCAL_DOT3_AGG_ID 10
+#define LLDP_SNMP_LOCAL_DOT3_MFS 11
+#define LLDP_SNMP_LOCAL_DOT1_PVID 12
 /* Remote ports */
 #define LLDP_SNMP_REMOTE_CIDSUBTYPE 1
 #define LLDP_SNMP_REMOTE_CID 2
@@ -341,6 +343,8 @@ header_tprvindexed_table(struct variable *vp, oid *name, size_t *length,
 #define LLDP_SNMP_REMOTE_DOT3_AUTONEG_MAU 13
 #define LLDP_SNMP_REMOTE_DOT3_AGG_STATUS 14
 #define LLDP_SNMP_REMOTE_DOT3_AGG_ID 15
+#define LLDP_SNMP_REMOTE_DOT3_MFS 16
+#define LLDP_SNMP_REMOTE_DOT1_PVID 17
 /* Local vlans */
 #define LLDP_SNMP_LOCAL_DOT1_VLANNAME 1
 /* Remote vlans */
@@ -529,10 +533,10 @@ agent_h_remote_med(struct variable *vp, oid *name, size_t *length,
 		return (u_char *)&bit;
 	case LLDP_SNMP_MED_REMOTE_CAP_ENABLED:
 		*var_len = 1;
-		bit = swap_bits(hardware->h_rchassis->c_med_cap_enabled);
+		bit = swap_bits(hardware->h_rport->p_med_cap_enabled);
 		return (u_char *)&bit;
 	case LLDP_SNMP_MED_REMOTE_POE_DEVICETYPE:
-		switch (hardware->h_rchassis->c_med_pow_devicetype) {
+		switch (hardware->h_rport->p_med_pow_devicetype) {
 		case LLDPMED_POW_TYPE_PSE:
 			long_ret = 2; break;
 		case LLDPMED_POW_TYPE_PD:
@@ -546,19 +550,19 @@ agent_h_remote_med(struct variable *vp, oid *name, size_t *length,
 	case LLDP_SNMP_MED_REMOTE_POE_PSE_POWERVAL:
 	case LLDP_SNMP_MED_REMOTE_POE_PD_POWERVAL:
 		if (((vp->magic == LLDP_SNMP_MED_REMOTE_POE_PSE_POWERVAL) &&
-			(hardware->h_rchassis->c_med_pow_devicetype ==
+			(hardware->h_rport->p_med_pow_devicetype ==
 			LLDPMED_POW_TYPE_PSE)) ||
 		    ((vp->magic == LLDP_SNMP_MED_REMOTE_POE_PD_POWERVAL) &&
-			(hardware->h_rchassis->c_med_pow_devicetype ==
+			(hardware->h_rport->p_med_pow_devicetype ==
 			    LLDPMED_POW_TYPE_PD))) {
-			long_ret = hardware->h_rchassis->c_med_pow_val;
+			long_ret = hardware->h_rport->p_med_pow_val;
 			return (u_char *)&long_ret;
 		}
 		break;
 	case LLDP_SNMP_MED_REMOTE_POE_PSE_POWERSOURCE:
-		if (hardware->h_rchassis->c_med_pow_devicetype ==
+		if (hardware->h_rport->p_med_pow_devicetype ==
 		    LLDPMED_POW_TYPE_PSE) {
-			switch (hardware->h_rchassis->c_med_pow_source) {
+			switch (hardware->h_rport->p_med_pow_source) {
 			case LLDPMED_POW_SOURCE_PRIMARY:
 				long_ret = 2; break;
 			case LLDPMED_POW_SOURCE_BACKUP:
@@ -570,9 +574,9 @@ agent_h_remote_med(struct variable *vp, oid *name, size_t *length,
 		}
 		break;
 	case LLDP_SNMP_MED_REMOTE_POE_PD_POWERSOURCE:
-		if (hardware->h_rchassis->c_med_pow_devicetype ==
+		if (hardware->h_rport->p_med_pow_devicetype ==
 		    LLDPMED_POW_TYPE_PD) {
-			switch (hardware->h_rchassis->c_med_pow_source) {
+			switch (hardware->h_rport->p_med_pow_source) {
 			case LLDPMED_POW_SOURCE_PSE:
 				long_ret = 2; break;
 			case LLDPMED_POW_SOURCE_LOCAL:
@@ -588,12 +592,12 @@ agent_h_remote_med(struct variable *vp, oid *name, size_t *length,
 	case LLDP_SNMP_MED_REMOTE_POE_PSE_POWERPRIORITY:
 	case LLDP_SNMP_MED_REMOTE_POE_PD_POWERPRIORITY:
 		if (((vp->magic == LLDP_SNMP_MED_REMOTE_POE_PSE_POWERPRIORITY) &&
-			(hardware->h_rchassis->c_med_pow_devicetype ==
+			(hardware->h_rport->p_med_pow_devicetype ==
 			LLDPMED_POW_TYPE_PSE)) ||
 		    ((vp->magic == LLDP_SNMP_MED_REMOTE_POE_PD_POWERPRIORITY) &&
-			(hardware->h_rchassis->c_med_pow_devicetype ==
+			(hardware->h_rport->p_med_pow_devicetype ==
 			    LLDPMED_POW_TYPE_PD))) {
-			switch (hardware->h_rchassis->c_med_pow_priority) {
+			switch (hardware->h_rport->p_med_pow_priority) {
 			case LLDPMED_POW_PRIO_CRITICAL:
 				long_ret = 2; break;
 			case LLDPMED_POW_PRIO_HIGH:
@@ -665,7 +669,7 @@ agent_h_remote_med_policy(struct variable *vp, oid *name, size_t *length,
 	type = name[*length - 1];
 	if ((type < 1) || (type > LLDPMED_APPTYPE_LAST))
 		goto remotemedpolicy_failed;
-	policy = &hardware->h_rchassis->c_med_policy[type-1];
+	policy = &hardware->h_rport->p_med_policy[type-1];
 	if (policy->type != type)
 		goto remotemedpolicy_failed;
 
@@ -717,7 +721,7 @@ agent_h_remote_med_location(struct variable *vp, oid *name, size_t *length,
 	type = name[*length - 1];
 	if ((type < 1) || (type > LLDPMED_APPTYPE_LAST))
 		goto remotemedlocation_failed;
-	location = &hardware->h_rchassis->c_med_location[type-1];
+	location = &hardware->h_rport->p_med_location[type-1];
 	if (location->format != type)
 		goto remotemedlocation_failed;
 
@@ -857,6 +861,14 @@ agent_h_local_port(struct variable *vp, oid *name, size_t *length,
         case LLDP_SNMP_LOCAL_DOT3_AGG_ID:
                 long_ret = hardware->h_lport.p_aggregid;
                 return (u_char *)&long_ret;
+	case LLDP_SNMP_LOCAL_DOT3_MFS:
+		long_ret = hardware->h_lport.p_mfs;
+		return (u_char *)&long_ret;
+#endif
+#ifdef ENABLE_DOT1
+	case LLDP_SNMP_LOCAL_DOT1_PVID:
+		long_ret = hardware->h_lport.p_pvid; /* Should always be 0 */
+		return (u_char *)&long_ret;
 #endif
 	default:
 		break;
@@ -968,6 +980,14 @@ agent_h_remote_port(struct variable *vp, oid *name, size_t *length,
         case LLDP_SNMP_REMOTE_DOT3_AGG_ID:
                 long_ret = hardware->h_rport->p_aggregid;
                 return (u_char *)&long_ret;
+        case LLDP_SNMP_REMOTE_DOT3_MFS:
+                long_ret = hardware->h_rport->p_mfs;
+                return (u_char *)&long_ret;
+#endif
+#ifdef ENABLE_DOT1
+        case LLDP_SNMP_REMOTE_DOT1_PVID:
+                long_ret = hardware->h_rport->p_pvid;
+                return (u_char *)&long_ret;
 #endif
 	default:
 		break;
@@ -1104,6 +1124,12 @@ static struct variable8 lldp_vars[] = {
          {1, 5, 4623, 1, 2, 3, 1, 1}},
         {LLDP_SNMP_LOCAL_DOT3_AGG_ID, ASN_INTEGER, RONLY, agent_h_local_port, 8,
          {1, 5, 4623, 1, 2, 3, 1, 2}},
+        {LLDP_SNMP_LOCAL_DOT3_MFS, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+         {1, 5, 4623, 1, 2, 4, 1, 1}},
+#endif
+#ifdef ENABLE_DOT1
+        {LLDP_SNMP_LOCAL_DOT1_PVID, ASN_INTEGER, RONLY, agent_h_local_port, 8,
+         {1, 5, 32962, 1, 2, 1, 1, 1}},
 #endif
         /* Remote ports */
         {LLDP_SNMP_REMOTE_CIDSUBTYPE, ASN_INTEGER, RONLY, agent_h_remote_port, 5, {1, 4, 1, 1, 4}},
@@ -1128,8 +1154,12 @@ static struct variable8 lldp_vars[] = {
          {1, 5, 4623, 1, 3, 3, 1, 1}},
         {LLDP_SNMP_REMOTE_DOT3_AGG_ID, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
          {1, 5, 4623, 1, 3, 3, 1, 2}},
+        {LLDP_SNMP_REMOTE_DOT3_MFS, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+         {1, 5, 4623, 1, 3, 4, 1, 1}},
 #endif
 #ifdef ENABLE_DOT1
+        {LLDP_SNMP_REMOTE_DOT1_PVID, ASN_INTEGER, RONLY, agent_h_remote_port, 8,
+         {1, 5, 32962, 1, 3, 1, 1, 1}},
         /* Local vlans */
         {LLDP_SNMP_LOCAL_DOT1_VLANNAME, ASN_OCTET_STR, RONLY, agent_h_local_vlan, 8,
          {1, 5, 32962, 1, 2, 3, 1, 2}},
