@@ -46,13 +46,13 @@
 #include <net-snmp/agent/snmp_vars.h>
 #endif /* USE_SNMP */
 
-void		 usage(void);
+static void		 usage(void);
 
-int			 lldpd_iface_init(struct lldpd *, struct lldpd_hardware *);
-int			 lldpd_iface_init_vlan(struct lldpd *, struct lldpd_vif *);
-void			 lldpd_iface_init_mtu(struct lldpd *, struct lldpd_hardware *);
-int			 lldpd_iface_close(struct lldpd *, struct lldpd_hardware *);
-void			 lldpd_iface_multicast(struct lldpd *, const char *, int);
+static int		 lldpd_iface_init(struct lldpd *, struct lldpd_hardware *);
+static int		 lldpd_iface_init_vlan(struct lldpd *, struct lldpd_vif *);
+static void		 lldpd_iface_init_mtu(struct lldpd *, struct lldpd_hardware *);
+static int		 lldpd_iface_close(struct lldpd *, struct lldpd_hardware *);
+static void		 lldpd_iface_multicast(struct lldpd *, const char *, int);
 
 /* "ether proto 0x88cc and ether dst 01:80:c2:00:00:0e" */
 #define LLDPD_FILTER_LLDP_F						\
@@ -64,7 +64,7 @@ void			 lldpd_iface_multicast(struct lldpd *, const char *, int);
         { 0x15, 0, 1, 0x00000180 },					\
         { 0x6, 0, 0, 0x0000ffff },					\
         { 0x6, 0, 0, 0x00000000 },
-struct sock_filter lldpd_filter_lldp_f[] = { LLDPD_FILTER_LLDP_F };
+static struct sock_filter lldpd_filter_lldp_f[] = { LLDPD_FILTER_LLDP_F };
 #ifdef ENABLE_FDP
 /* "ether dst 01:e0:52:cc:cc:cc" */
 #define LLDPD_FILTER_FDP_F			\
@@ -74,7 +74,7 @@ struct sock_filter lldpd_filter_lldp_f[] = { LLDPD_FILTER_LLDP_F };
         { 0x15, 0, 1, 0x000001e0 },		\
         { 0x6, 0, 0, 0x0000ffff },		\
         { 0x6, 0, 0, 0x00000000 },
-struct sock_filter lldpd_filter_fdp_f[] = { LLDPD_FILTER_FDP_F };
+static struct sock_filter lldpd_filter_fdp_f[] = { LLDPD_FILTER_FDP_F };
 #endif /* ENABLE_FDP */
 #ifdef ENABLE_CDP
 /* "ether dst 01:00:0c:cc:cc:cc" */
@@ -85,7 +85,7 @@ struct sock_filter lldpd_filter_fdp_f[] = { LLDPD_FILTER_FDP_F };
         { 0x15, 0, 1, 0x00000100 },		\
         { 0x6, 0, 0, 0x0000ffff },		\
         { 0x6, 0, 0, 0x00000000 },
-struct sock_filter lldpd_filter_cdp_f[] = { LLDPD_FILTER_CDP_F };
+static struct sock_filter lldpd_filter_cdp_f[] = { LLDPD_FILTER_CDP_F };
 #endif /* ENABLE_CDP */
 #ifdef ENABLE_SONMP
 /* "ether dst 01:00:81:00:01:00" */
@@ -96,7 +96,7 @@ struct sock_filter lldpd_filter_cdp_f[] = { LLDPD_FILTER_CDP_F };
         { 0x15, 0, 1, 0x00000100 },		\
         { 0x6, 0, 0, 0x0000ffff },		\
         { 0x6, 0, 0, 0x00000000 },
-struct sock_filter lldpd_filter_sonmp_f[] = { LLDPD_FILTER_SONMP_F };
+static struct sock_filter lldpd_filter_sonmp_f[] = { LLDPD_FILTER_SONMP_F };
 #endif /* ENABLE_SONMP */
 #ifdef ENABLE_EDP
 /* "ether dst 00:e0:2b:00:00:00" */
@@ -107,8 +107,8 @@ struct sock_filter lldpd_filter_sonmp_f[] = { LLDPD_FILTER_SONMP_F };
 	{ 0x15, 0, 1, 0x000000e0 },	\
 	{ 0x6, 0, 0, 0x0000ffff },	\
 	{ 0x6, 0, 0, 0x00000000 },
+static struct sock_filter lldpd_filter_edp_f[] = { LLDPD_FILTER_EDP_F };
 #endif /* ENABLE_EDP */
-struct sock_filter lldpd_filter_edp_f[] = { LLDPD_FILTER_EDP_F };
 #define LLDPD_FILTER_ANY_F		\
 	{ 0x28, 0, 0, 0x0000000c },	\
 	{ 0x15, 0, 4, 0x000088cc },	\
@@ -129,9 +129,9 @@ struct sock_filter lldpd_filter_edp_f[] = { LLDPD_FILTER_EDP_F };
 	{ 0x15, 0, 1, 0x000001e0 },	\
 	{ 0x6, 0, 0, 0x0000ffff },	\
 	{ 0x6, 0, 0, 0x00000000 },
-struct sock_filter lldpd_filter_any_f[] = { LLDPD_FILTER_ANY_F };
+static struct sock_filter lldpd_filter_any_f[] = { LLDPD_FILTER_ANY_F };
 
-struct protocol protos[] =
+static struct protocol protos[] =
 {
 	{ LLDPD_MODE_LLDP, 1, "LLDP", ' ', lldp_send, lldp_decode, NULL,
 	  LLDP_MULTICAST_ADDR, lldpd_filter_lldp_f, sizeof(lldpd_filter_lldp_f) },
@@ -157,24 +157,25 @@ struct protocol protos[] =
 	  {0,0,0,0,0,0}, lldpd_filter_any_f, sizeof(lldpd_filter_any_f) }
 };
 
-int			 lldpd_iface_switchto(struct lldpd *, short int,
+static int		 lldpd_iface_switchto(struct lldpd *, short int,
 			    struct lldpd_hardware *);
+static
 struct lldpd_hardware	*lldpd_port_add(struct lldpd *, struct ifaddrs *);
-void			 lldpd_loop(struct lldpd *);
-void			 lldpd_shutdown(int);
-void			 lldpd_exit();
-void			 lldpd_send_all(struct lldpd *);
-void			 lldpd_recv_all(struct lldpd *);
-int			 lldpd_guess_type(struct lldpd *, char *, int);
-void			 lldpd_decode(struct lldpd *, char *, int,
+static void		 lldpd_loop(struct lldpd *);
+static void		 lldpd_shutdown(int);
+static void		 lldpd_exit();
+static void		 lldpd_send_all(struct lldpd *);
+static void		 lldpd_recv_all(struct lldpd *);
+static int		 lldpd_guess_type(struct lldpd *, char *, int);
+static void		 lldpd_decode(struct lldpd *, char *, int,
 			    struct lldpd_hardware *, int);
 #ifdef ENABLE_LLDPMED
-void			 lldpd_med(struct lldpd_chassis *);
+static void		 lldpd_med(struct lldpd_chassis *);
 #endif
 
-char	**saved_argv;
+static char		**saved_argv;
 
-void
+static void
 usage(void)
 {
 	extern const char	*__progname;
@@ -183,7 +184,7 @@ usage(void)
 	exit(1);
 }
 
-void
+static void
 lldpd_iface_init_mtu(struct lldpd *global, struct lldpd_hardware *hardware)
 {
 	struct ifreq ifr;
@@ -195,10 +196,10 @@ lldpd_iface_init_mtu(struct lldpd *global, struct lldpd_hardware *hardware)
 		LLOG_WARN("unable to get MTU of %s, using 1500", hardware->h_ifname);
 		hardware->h_mtu = 1500;
 	} else
-		hardware->h_mtu = ifr.ifr_mtu;
+		hardware->h_mtu = hardware->h_lport.p_mfs = ifr.ifr_mtu;
 }
 
-int
+static int
 lldpd_iface_init_vlan(struct lldpd *global, struct lldpd_vif *vif)
 {
 	int status;
@@ -227,7 +228,7 @@ lldpd_iface_init_vlan(struct lldpd *global, struct lldpd_vif *vif)
 	return 0;
 }
 
-int
+static int
 lldpd_iface_init(struct lldpd *global, struct lldpd_hardware *hardware)
 {
 	int master;		/* Bond device */
@@ -277,7 +278,7 @@ lldpd_iface_init(struct lldpd *global, struct lldpd_hardware *hardware)
 	return 0;
 }
 
-void
+static void
 lldpd_iface_multicast(struct lldpd *global, const char *name, int remove)
 {
 	int i, rc;
@@ -296,7 +297,7 @@ lldpd_iface_multicast(struct lldpd *global, const char *name, int remove)
 	}
 }
 
-int
+static int
 lldpd_iface_close(struct lldpd *global, struct lldpd_hardware *hardware)
 {
 	char listen[IFNAMSIZ];
@@ -313,14 +314,14 @@ lldpd_iface_close(struct lldpd *global, struct lldpd_hardware *hardware)
 		close(hardware->h_raw_real);
 		lldpd_iface_multicast(global, listen, 1);
 	}
-	strlcpy(listen, hardware->h_ifname, sizeof(listen));
+	memcpy(listen, hardware->h_ifname, IFNAMSIZ);
 	lldpd_iface_multicast(global, listen, 1);
 
 	hardware->h_raw_real = -1;
 	return 0;
 }
 
-int
+static int
 lldpd_iface_switchto(struct lldpd *cfg, short int filter, struct lldpd_hardware *hardware)
 {
 	struct sock_fprog prog;
@@ -364,30 +365,38 @@ lldpd_vlan_cleanup(struct lldpd_port *port)
 }
 #endif
 
+/* If `all' is true, clear all information, including information that
+   are not refreshed periodically. If `all' is true, also free the
+   port. */
 void
-lldpd_port_cleanup(struct lldpd_port *port)
+lldpd_port_cleanup(struct lldpd_port *port, int all)
 {
+#ifdef ENABLE_LLDPMED
+	int i;
+	if (all)
+		for (i=0; i < LLDPMED_LOCFORMAT_LAST; i++)
+			free(port->p_med_location[i].data);
+#endif
 #ifdef ENABLE_DOT1
 	lldpd_vlan_cleanup(port);
 #endif
 	free(port->p_id);
 	free(port->p_descr);
-	free(port);
+	if (all)
+		free(port);
 }
 
 void
 lldpd_chassis_cleanup(struct lldpd_chassis *chassis)
 {
 #ifdef ENABLE_LLDPMED
-	int i;
 	free(chassis->c_med_hw);
+	free(chassis->c_med_sw);
 	free(chassis->c_med_fw);
 	free(chassis->c_med_sn);
 	free(chassis->c_med_manuf);
 	free(chassis->c_med_model);
 	free(chassis->c_med_asset);
-	for (i=0; i < LLDPMED_LOCFORMAT_LAST; i++)
-		free(chassis->c_med_location[i].data);
 #endif
 	free(chassis->c_id);
 	free(chassis->c_name);
@@ -399,7 +408,7 @@ void
 lldpd_remote_cleanup(struct lldpd *cfg, struct lldpd_hardware *hardware, int reset)
 {
 	if (hardware->h_rport != NULL) {
-		lldpd_port_cleanup(hardware->h_rport);
+		lldpd_port_cleanup(hardware->h_rport, 1);
 		hardware->h_rport = NULL;
 	}
 	if (hardware->h_rchassis != NULL) {
@@ -420,9 +429,7 @@ lldpd_remote_cleanup(struct lldpd *cfg, struct lldpd_hardware *hardware, int res
 void
 lldpd_hardware_cleanup(struct lldpd_hardware *hardware)
 {
-#ifdef ENABLE_DOT1
-	lldpd_vlan_cleanup(&hardware->h_lport);
-#endif
+	lldpd_port_cleanup(&hardware->h_lport, 1);
 	free(hardware->h_proto_macs);
 	free(hardware->h_llastframe);
 	free(hardware);
@@ -440,7 +447,7 @@ lldpd_cleanup(struct lldpd *cfg)
 		if (hardware->h_flags == 0) {
 			TAILQ_REMOVE(&cfg->g_hardware, hardware, h_entries);
 			lldpd_iface_close(cfg, hardware);
-			lldpd_remote_cleanup(cfg, hardware, 1);
+			lldpd_remote_cleanup(cfg, hardware, 0);
 			lldpd_hardware_cleanup(hardware);
 		} else if (hardware->h_rchassis != NULL) {
 			if (time(NULL) - hardware->h_rlastupdate >
@@ -461,7 +468,7 @@ lldpd_cleanup(struct lldpd *cfg)
 	}
 }
 
-struct lldpd_vif *
+static struct lldpd_vif *
 lldpd_port_add_vlan(struct lldpd *cfg, struct ifaddrs *ifa)
 {
 	struct lldpd_vif *vif;
@@ -515,7 +522,7 @@ lldpd_port_add_vlan(struct lldpd *cfg, struct ifaddrs *ifa)
 	return vif;
 }
 
-struct lldpd_hardware *
+static struct lldpd_hardware *
 lldpd_port_add(struct lldpd *cfg, struct ifaddrs *ifa)
 {
 #if defined (ENABLE_DOT1) || defined (ENABLE_DOT3)
@@ -545,10 +552,15 @@ lldpd_port_add(struct lldpd *cfg, struct ifaddrs *ifa)
 		hardware->h_raw_real = -1;
 		hardware->h_start_probe = 0;
 		hardware->h_proto_macs = (u_int8_t*)calloc(cfg->g_multi+1, ETH_ALEN);
+#ifdef ENABLE_LLDPMED
+		hardware->h_lport.p_med_cap_enabled = LLDPMED_CAP_CAP;
+		if (!cfg->g_noinventory)
+			hardware->h_lport.p_med_cap_enabled |= LLDPMED_CAP_IV;
+#endif
 #ifdef ENABLE_DOT1
 		TAILQ_INIT(&hardware->h_lport.p_vlans);
 	} else {
-		lldpd_vlan_cleanup(&hardware->h_lport);
+		lldpd_port_cleanup(&hardware->h_lport, 0);
 #endif
 	}
 
@@ -564,7 +576,7 @@ lldpd_port_add(struct lldpd *cfg, struct ifaddrs *ifa)
 		fatal(NULL);
 	memcpy(port->p_id, hardware->h_lladdr, sizeof(hardware->h_lladdr));
 	port->p_id_len = sizeof(hardware->h_lladdr);
-	port->p_descr = hardware->h_ifname;
+	port->p_descr = strdup(hardware->h_ifname);
 
 	if (cfg->g_lchassis.c_id == NULL) {
 		/* Use the first port's l2 addr as the chassis ID */
@@ -585,8 +597,10 @@ lldpd_port_add(struct lldpd *cfg, struct ifaddrs *ifa)
 		/* Check if we already have checked this one */
 		int skip = 0;
 		TAILQ_FOREACH(vlan, &port->p_vlans, v_entries) {
-			if (strcmp(vlan->v_name, oifa->ifa_name) == 0)
+			if (strcmp(vlan->v_name, oifa->ifa_name) == 0) {
 				skip = 1;
+				break;
+			}
 		}
 		if (skip) continue;
 #endif
@@ -604,6 +618,7 @@ lldpd_port_add(struct lldpd *cfg, struct ifaddrs *ifa)
 		strlcpy(ifv.device1, oifa->ifa_name, sizeof(ifv.device1));
 		if ((ioctl(cfg->g_sock, SIOCGIFVLAN, &ifv) >= 0) &&
 		    ((iface_is_bond_slave(cfg, hardware->h_ifname, ifv.u.device2, NULL)) ||
+		     (iface_is_bridged_to(cfg, hardware->h_ifname, ifv.u.device2)) ||
 		     (strncmp(hardware->h_ifname, ifv.u.device2, sizeof(ifv.u.device2)) == 0))) {
 			if ((vlan = (struct lldpd_vlan *)
 			     calloc(1, sizeof(struct lldpd_vlan))) == NULL)
@@ -702,7 +717,7 @@ lldpd_port_add(struct lldpd *cfg, struct ifaddrs *ifa)
 	return (hardware);
 }
 
-int
+static int
 lldpd_guess_type(struct lldpd *cfg, char *frame, int s)
 {
 	int i;
@@ -722,7 +737,7 @@ lldpd_guess_type(struct lldpd *cfg, char *frame, int s)
 	return -1;
 }
 
-void
+static void
 lldpd_decode(struct lldpd *cfg, char *frame, int s,
     struct lldpd_hardware *hardware, int bond)
 {
@@ -735,8 +750,8 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
 	int guess = LLDPD_MODE_LLDP;
 
 	/* Discard VLAN frames */
-	if ((s >= sizeof(struct ieee8023)) &&
-	    (((struct ieee8023*)frame)->size == htons(ETHERTYPE_VLAN)))
+	if ((s >= sizeof(struct ethhdr)) &&
+	    (((struct ethhdr*)frame)->h_proto == htons(ETHERTYPE_VLAN)))
 		return;
 
 	if ((hardware->h_rlastframe != NULL) &&
@@ -966,11 +981,11 @@ lldpd_decode(struct lldpd *cfg, char *frame, int s,
 
 cleanup:
 	lldpd_chassis_cleanup(chassis);
-	lldpd_port_cleanup(port);
+	lldpd_port_cleanup(port, 1);
 	return;
 }
 
-void
+static void
 lldpd_recv_all(struct lldpd *cfg)
 {
 	struct lldpd_hardware *hardware;
@@ -1001,7 +1016,8 @@ lldpd_recv_all(struct lldpd *cfg)
 		
 		TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
 			/* Ignore if interface is down */
-			if ((hardware->h_flags & IFF_UP) == 0)
+			if (((hardware->h_flags & IFF_UP) == 0) ||
+			    ((hardware->h_flags & IFF_RUNNING) == 0))
 				continue;
 			FD_SET(hardware->h_raw, &rfds);
 			if (nfds < hardware->h_raw)
@@ -1015,7 +1031,8 @@ lldpd_recv_all(struct lldpd *cfg)
 			}
 		}
 		TAILQ_FOREACH(vif, &cfg->g_vif, vif_entries) {
-			if ((vif->vif_flags & IFF_UP) == 0)
+			if (((vif->vif_flags & IFF_UP) == 0) ||
+			    ((vif->vif_flags & IFF_RUNNING) == 0))
 				continue;
 			FD_SET(vif->vif_raw, &rfds);
 			if (nfds < vif->vif_raw)
@@ -1179,7 +1196,7 @@ lldpd_recv_all(struct lldpd *cfg)
 	} while ((rc != 0) || (time(NULL) - cfg->g_lastsent < cfg->g_delay));
 }
 
-void
+static void
 lldpd_send_all(struct lldpd *cfg)
 {
 	struct lldpd_hardware *hardware;
@@ -1189,7 +1206,8 @@ lldpd_send_all(struct lldpd *cfg)
 	cfg->g_lastsent = time(NULL);
 	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
 		/* Ignore if interface is down */
-		if ((hardware->h_flags & IFF_UP) == 0)
+		if (((hardware->h_flags & IFF_UP) == 0) ||
+		    ((hardware->h_flags & IFF_RUNNING) == 0))
 			continue;
 
 		/* When sending on inactive slaves, just send using a 0:0:0:0:0:0 address */
@@ -1216,7 +1234,7 @@ lldpd_send_all(struct lldpd *cfg)
 }
 
 #ifdef ENABLE_LLDPMED
-void
+static void
 lldpd_med(struct lldpd_chassis *chassis)
 {
 	free(chassis->c_med_hw);
@@ -1234,7 +1252,7 @@ lldpd_med(struct lldpd_chassis *chassis)
 }
 #endif
 
-void
+static void
 lldpd_loop(struct lldpd *cfg)
 {
 	struct ifaddrs *ifap, *ifa;
@@ -1332,7 +1350,7 @@ lldpd_loop(struct lldpd *cfg)
 		    (iface_is_bond(cfg, ifa->ifa_name)))
 			continue;
 
-                if (!(ifa->ifa_flags & IFF_MULTICAST))
+                if (!(ifa->ifa_flags & (IFF_MULTICAST|IFF_BROADCAST)))
                         continue;
 
 		if (iface_is_wireless(cfg, ifa->ifa_name))
@@ -1362,189 +1380,7 @@ lldpd_loop(struct lldpd *cfg)
 	lldpd_recv_all(cfg);
 }
 
-#ifdef ENABLE_LLDPMED
-void
-lldpd_parse_location(struct lldpd_chassis *chassis, const char *location)
-{
-	char *l, *e, *s, *data, *n;
-	double ll, altitude;
-	u_int32_t intpart, floatpart;
-	int type = 0, i;
-
-	if ((l = strdup(location)) == NULL)
-		fatal(NULL);
-	s = l;
-	if ((e = index(s, ':')) == NULL)
-		goto invalid_location;
-	*e = '\0';
-	type = atoi(s);
-	switch (type) {
-	case LLDPMED_LOCFORMAT_COORD:
-		/* Coordinates */
-		if ((chassis->c_med_location[0].data =
-			(char *)malloc(16)) == NULL)
-			fatal(NULL);
-		chassis->c_med_location[0].data_len = 16;
-		chassis->c_med_location[0].format = LLDPMED_LOCFORMAT_COORD;
-		data = chassis->c_med_location[0].data;
-
-		/* Latitude and longitude */
-		for (i = 0; i < 2; i++) {
-			s = e+1;
-			if ((e = index(s, ':')) == NULL)
-				goto invalid_location;
-			*e = '\0';
-			ll = atof(s);
-			s = e + 1;
-			if ((e = index(s, ':')) == NULL)
-				goto invalid_location;
-			*e = '\0';
-			intpart = (int)ll;
-			floatpart = (ll - intpart) * (1 << 25);
-			if (((i == 0) && (*s == 'S')) ||
-			    ((i == 1) && (*s == 'W'))) {
-				intpart = ~intpart;
-				intpart += 1;
-				floatpart = ~floatpart;
-				floatpart += 1;
-			} else if (((i == 0) && (*s != 'N')) ||
-			    ((i == 1) && (*s != 'E'))) 
-				goto invalid_location;
-			*(u_int8_t *)data = (6 << 2) |	       /* Precision */
-			    ((intpart & 0x180) >> 7);	       /* Int part 2 bits */
-			data++;
-			*(u_int8_t *)data = (((intpart & 0x7f) << 1) | /* Int part 7 bits */
-			    ((floatpart & 0x1000000) >> 24));	/* Float part 1 bit */
-			data++;
-			*(u_int8_t *)data = (floatpart & 0xff0000) >> 16; /* 8 bits */
-			data++;
-			*(u_int8_t *)data = (floatpart & 0xff00) >> 8; /* 8 bits */
-			data++;
-			*(u_int8_t *)data = (floatpart & 0xff); /* 8 bits */
-			data++;
-		}
-		
-		/* Altitude */
-		s = e+1;
-		if ((e = index(s, ':')) == NULL)
-			goto invalid_location;
-		*e = '\0';
-		altitude = atof(s);
-		s = e+1;
-		if ((e = index(s, ':')) == NULL)
-			goto invalid_location;
-		*e = '\0';
-		if (altitude < 0) {
-			intpart = -(int)altitude;
-			floatpart = (-(altitude + intpart)) * (1 << 8);
-			intpart = ~intpart; intpart += 1;
-			floatpart = ~floatpart; floatpart += 1;
-		} else {
-			intpart = (int)altitude;
-			floatpart = (altitude - intpart) * (1 << 8);
-		}
-		if ((*s != 'm') && (*s != 'f'))
-			goto invalid_location;
-		*(u_int8_t *)data = ((((*s == 'm')?1:2) << 4) |	       /* Type 4 bits */
-		    0);						       /* Precision 4 bits */
-		data++;
-		*(u_int8_t *)data = ((6 << 6) |			       /* Precision 2 bits */
-		    ((intpart & 0x3f0000) >> 16));		       /* Int 6 bits */
-		data++;
-		*(u_int8_t *)data = (intpart & 0xff00) >> 8; /* Int 8 bits */
-		data++;
-		*(u_int8_t *)data = intpart & 0xff; /* Int 8 bits */
-		data++;
-		*(u_int8_t *)data = floatpart & 0xff; /* Float 8 bits */
-		data++;
-
-		/* Datum */
-		s = e + 1;
-		if (index(s, ':') != NULL)
-			goto invalid_location;
-		*(u_int8_t *)data = atoi(s);
-		break;
-	case LLDPMED_LOCFORMAT_CIVIC:
-		/* Civic address */
-		chassis->c_med_location[1].data_len = 4;
-		s = e+1;
-		if ((s = index(s, ':')) == NULL)
-			goto invalid_location;
-		s = s+1;
-		do {
-			if ((s = index(s, ':')) == NULL)
-				break;
-			s = s+1;
-			/* s is the beginning of the word */
-			if ((n = index(s, ':')) == NULL)
-				n = s + strlen(s);
-			/* n is the end of the word */
-			chassis->c_med_location[1].data_len += (n - s) + 2;
-			if ((s = index(s, ':')) == NULL)
-				break;
-			s = s+1;
-		} while (1);
-		s = e+1;
-		if ((chassis->c_med_location[1].data =
-			(char *)malloc(chassis->c_med_location[1].data_len)) ==
-		    NULL)
-			fatal(NULL);
-		chassis->c_med_location[1].format = LLDPMED_LOCFORMAT_CIVIC;
-		data = chassis->c_med_location[1].data;
-		*(u_int8_t *)data = chassis->c_med_location[1].data_len - 1;
-		data++;
-		*(u_int8_t *)data = 2; /* Client location */
-		data++;
-		if ((e = index(s, ':')) == NULL)
-			goto invalid_location;
-		if ((e - s) != 2)
-			goto invalid_location;
-		memcpy(data, s, 2); /* Country code */
-		data += 2;
-		while (*e != '\0') {
-			s=e+1;
-			if ((e = index(s, ':')) == NULL)
-				goto invalid_location;
-			*e = '\0';
-			*(u_int8_t *)data = atoi(s);
-			data++;
-			s=e+1;
-			if ((e = index(s, ':')) == NULL)
-				e = s + strlen(s);
-			*(u_int8_t *)data = e - s;
-			data++;
-			memcpy(data, s, e-s);
-			data += e-s;
-		}
-		break;
-	case LLDPMED_LOCFORMAT_ELIN:
-		s = e+1;
-		chassis->c_med_location[2].data_len = strlen(s);
-		if ((chassis->c_med_location[2].data =
-			(char *)malloc(strlen(s))) == NULL)
-			fatal(NULL);
-		chassis->c_med_location[2].format = LLDPMED_LOCFORMAT_ELIN;
-		strcpy(chassis->c_med_location[2].data, s);
-		break;
-	default:
-		goto invalid_location;
-	}
-
-	chassis->c_med_cap_enabled |= LLDPMED_CAP_LOCATION;
-	return;
-invalid_location:
-	LLOG_WARNX("the format of the location is invalid (%s)",
-		location);
-	if (type) {
-		free(chassis->c_med_location[type-1].data);
-		memset(&chassis->c_med_location[type-1], 0,
-		    sizeof(struct lldpd_med_loc));
-	}
-	free(l);
-}
-#endif
-
-void
+static void
 lldpd_shutdown(int sig)
 {
 	LLOG_INFO("signal received, exiting");
@@ -1552,9 +1388,9 @@ lldpd_shutdown(int sig)
 }
 
 /* For signal handling */
-struct lldpd *gcfg = NULL;
+static struct lldpd *gcfg = NULL;
 
-void
+static void
 lldpd_exit()
 {
 	struct lldpd_hardware *hardware;
@@ -1584,7 +1420,7 @@ main(int argc, char *argv[])
 	int snmp = 0;
 #endif
 	char *mgmtp = NULL;
-	char *popt, opts[] = "vdxm:p:M:iL:@                    ";
+	char *popt, opts[] = "vdxm:p:M:i@                    ";
 	int probe = 0, i, found, vlan = 0;
 #ifdef ENABLE_LLDPMED
 	int lldpmed = 0, noinventory = 0;
@@ -1623,13 +1459,9 @@ main(int argc, char *argv[])
 		case 'i':
 			noinventory = 1;
 			break;
-		case 'L':
-			/* Handled later */
-			break;
 #else
 		case 'M':
 		case 'i':
-		case 'L':
 		case 'P':
 			fprintf(stderr, "LLDP-MED support is not built-in\n");
 			usage();
@@ -1704,17 +1536,7 @@ main(int argc, char *argv[])
 		cfg->g_lchassis.c_med_type = lldpmed;
 		cfg->g_lchassis.c_med_cap_available = LLDPMED_CAP_CAP |
 		    LLDPMED_CAP_IV | LLDPMED_CAP_LOCATION;
-		cfg->g_lchassis.c_med_cap_enabled = LLDPMED_CAP_CAP;
-		if (!noinventory)
-			cfg->g_lchassis.c_med_cap_enabled |= LLDPMED_CAP_IV;
-		optind = 1;
-		while ((ch = getopt(argc, argv, opts)) != -1) {
-			switch (ch) {
-			case 'L':
-				lldpd_parse_location(&cfg->g_lchassis, optarg);
-				break;
-			}
-		}
+		cfg->g_noinventory = noinventory;
 	}
 #endif
 
